@@ -102,6 +102,7 @@ typedef enum {
 	HACKRF_VENDOR_REQUEST_GET_CLKIN_STATUS = 44,
 	HACKRF_VENDOR_REQUEST_BOARD_REV_READ = 45,
 	HACKRF_VENDOR_REQUEST_SUPPORTED_PLATFORM_READ = 46,
+	HACKRF_VENDOR_REQUEST_SET_LEDS = 47,
 } hackrf_vendor_request;
 
 #define USB_CONFIG_STANDARD 0x1
@@ -287,7 +288,7 @@ static int allocate_transfers(hackrf_device* const device)
 		uint32_t transfer_index;
 		device->transfers = (struct libusb_transfer**) calloc(
 			TRANSFER_COUNT,
-			sizeof(struct libusb_transfer));
+			sizeof(struct libusb_transfer*));
 		if (device->transfers == NULL) {
 			return HACKRF_ERROR_NO_MEM;
 		}
@@ -1456,7 +1457,14 @@ int ADDCALL hackrf_set_freq_explicit(
 	uint8_t length;
 	int result;
 
-	if (if_freq_hz < 2150000000 || if_freq_hz > 2750000000) {
+	/*
+	 * Restriction to the range 2170-2740 MHz is strongly recommended for
+	 * HackRF One and Jawbreaker.  We permit IF as low as 2000 MHz and as
+	 * high as 3000 MHz for backwards compatibility and for
+	 * experimentation, but settings outside the recommended range are
+	 * unlikely to work.
+	 */
+	if (if_freq_hz < 2000000000 || if_freq_hz > 3000000000) {
 		return HACKRF_ERROR_INVALID_PARAM;
 	}
 
@@ -2196,6 +2204,26 @@ const char* ADDCALL hackrf_board_id_name(enum hackrf_board_id board_id)
 	}
 }
 
+extern ADDAPI uint32_t ADDCALL hackrf_board_id_platform(enum hackrf_board_id board_id)
+{
+	switch (board_id) {
+	case BOARD_ID_JAWBREAKER:
+		return HACKRF_PLATFORM_JAWBREAKER;
+
+	case BOARD_ID_HACKRF1_OG:
+		return HACKRF_PLATFORM_HACKRF1_OG;
+
+	case BOARD_ID_RAD1O:
+		return HACKRF_PLATFORM_RAD1O;
+
+	case BOARD_ID_HACKRF1_R9:
+		return HACKRF_PLATFORM_HACKRF1_R9;
+
+	default:
+		return 0;
+	}
+}
+
 extern ADDAPI const char* ADDCALL hackrf_usb_board_id_name(
 	enum hackrf_usb_board_id usb_board_id)
 {
@@ -2904,6 +2932,28 @@ int ADDCALL hackrf_supported_platform_read(hackrf_device* device, uint32_t* valu
 	}
 }
 
+int ADDCALL hackrf_set_leds(hackrf_device* device, const uint8_t state)
+{
+	USB_API_REQUIRED(device, 0x0107)
+	int result = libusb_control_transfer(
+		device->usb_device,
+		LIBUSB_ENDPOINT_OUT | LIBUSB_REQUEST_TYPE_VENDOR |
+			LIBUSB_RECIPIENT_DEVICE,
+		HACKRF_VENDOR_REQUEST_SET_LEDS,
+		state,
+		0,
+		NULL,
+		0,
+		0);
+
+	if (result != 0) {
+		last_libusb_error = result;
+		return HACKRF_ERROR_LIBUSB;
+	} else {
+		return HACKRF_SUCCESS;
+	}
+}
+
 #ifdef __cplusplus
 } // __cplusplus defined.
 #endif
